package service

import (
	"time"
)

type DataStore interface {
	WriteJournalEntry(interface{}) error
	GetJournalEntryForDate(time.Time) (interface{}, error)

	WritePlanDay(interface{}) error
	GetPlanDayForDate(time.Time) (interface{}, error)

	WritePlanWeek(interface{}) error
	GetPlanWeekForDate(time.Time) (interface{}, error)

	WritePlanMonth(interface{}) error
	GetPlanMonthForDate(time.Time) (interface{}, error)

	WriteTrackingItems(interface{}) error
	GetTrackingItemsForDate(time.Time) (interface{}, error)
	GetTrackingCategories() (interface{}, error)

	WriteInboxItem(interface{}) error
	DeleteInboxItem(int32) error
	GetInboxItems() ([]interface{}, error)
}
